<?php

namespace AOP\App\Options;

use AOP\App\Data;
use AOP\App\Admin\AdminPages\SubpageEdit;
use AOP\App\Admin\AdminPages\Settings\SubpageSettings;

class SingleOptionsPage
{
    private $parentSlug;
    private $subpageTitle;
    private $menuTitle;
    private $capability;
    private $menuSlug;
    private $hasTab;

    public function run($args)
    {
        $this->parentSlug = $args['parent_slug'];
        $this->menuTitle  = $args['menu_title'];
        $this->capability = $args['capability'];
        $this->menuSlug   = $args['menu_slug'];
        $this->hasTab     = $args['include_tab'];

        $this->subpageTitle = $args['select_title'] ? $args['select_title'] : $this->menuTitle;

        add_action('admin_menu', [$this, 'submenuPage']);
    }

    public function submenuPage()
    {
        add_submenu_page(
            $this->parentSlug,
            $this->subpageTitle,
            $this->menuTitle,
            $this->capability,
            $this->menuSlug,
            [$this, 'optionsDisplay']
        );
    }

    /**
     * Show the option page.
     *
     * @return mixed
     */
    public function optionsDisplay()
    {
        $submenuPages = Data::getSubpages();

        if (!$submenuPages) {
            return;
        }

        $activePage = $_GET['page'] ?: $this->menuSlug;

        print('<div class="wrap">');

        if ($_SERVER['DOCUMENT_URI'] !== '/wp-admin/options-general.php') {
            settings_errors();
        }

        $this->editButton();

        $this->showTitle($submenuPages);

        $this->showTabs($submenuPages, $activePage);

        $this->showForm($submenuPages, $activePage);

        print('</div>');
    }

    /**
     * Show the title if checked.
     *
     * @param object $submenuPages
     * @return mixed
     */
    private function showTitle($submenuPages)
    {
        $subpageHasCheckedTitle = $submenuPages->filter(function ($subpage) {
            return $subpage['menu_slug'] === $this->menuSlug && $subpage['select_title'] !== 'None';
        });

        if (!$subpageHasCheckedTitle->toArray()) {
            return;
        }

        printf('<h1>%s</h1>', $subpageHasCheckedTitle->pluck('select_title')->first());
    }

    /**
     * Show tabs if checked.
     *
     * @param object $submenuPages
     * @param string $activePage
     * @return mixed
     */
    private function showTabs($submenuPages, $activePage)
    {
        if (!$this->hasTab) {
            return;
        }

        $currentSubmenuPages = $submenuPages->filter(function ($subpage) {
            return $subpage['parent_slug'] === $this->parentSlug;
        })->values();

        print('<h2 class="nav-tab-wrapper">');

        $currentSubmenuPages->each(function ($subpage) use ($activePage, $currentSubmenuPages) {
            $activeClass = ($subpage['menu_slug'] === $activePage) ? ' nav-tab-active' : '';

            $symbol = strpos($subpage['parent_slug'], '?') ? '&amp;' : '?';

            $firstPage = $currentSubmenuPages[0];

            $parentSlugCheck = ($firstPage['parent_slug'] !== $firstPage['menu_slug']) ? $subpage['parent_slug'] : '';

            $slug = $parentSlugCheck . $symbol . 'page=' . $subpage['menu_slug'];

            printf(
                '<a href="%s" class="nav-tab%s">%s</a>',
                $slug,
                $activeClass,
                $subpage['menu_title']
            );
        });

        print('</h2>');
    }

    /**
     * Show the form with fields and submit button.
     *
     * @param object $submenuPages
     * @param string $activePage
     * @return mixed
     */
    private function showForm($submenuPages, $activePage)
    {
        $submenuPageHasFields = $submenuPages->filter(function ($subpage) use ($activePage) {
            return $subpage['menu_slug'] === $activePage && $subpage['fields'];
        });

        if (!$submenuPageHasFields->all()) {
            return;
        }

        print('<form method="post" action="options.php">');

        $submenuPageHasFields->map(function ($subpage) {
            settings_fields($subpage['menu_slug']);
            do_settings_sections($subpage['menu_slug']);
        });

        $fieldHasFieldName = $submenuPageHasFields->pluck('fields')
            ->flatten(1)
            ->filter(function ($field) {
                return isset($field['field_name']);
            })->all();

        if ($fieldHasFieldName) {
            submit_button();
        }

        print('</form>');
    }

    /**
     * Show the edit button on top right of the screen.
     *
     * @return void
     */
    private function editButton()
    {
        $isHidden = get_option(SubpageSettings::OPTION_1);

        if ($isHidden === '1') {
            return;
        }

        $id = (new Data())->getIdfromOptionPage($this->parentSlug);

        printf(
            '<a class="alignright button" href="%s?page=%s&optionpage=%s%s">%s</a>',
            admin_url() . 'admin.php',
            SubpageEdit::SLUG,
            $id,
            '#' . substr($this->menuSlug, -10),
            'Edit page'
        );
    }
}
